local toc, data = ...
local AddonId = toc.identifier

-- Local config options ---------------------------------------------------------
local nosheiFrameWidth = 260
local nosheiFrameHeight = 63
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local NosheiFrame = WT.UnitFrame:Template("NosheiFrame")
NosheiFrame.Configuration.Name = "Replacement of nkThings Target frame"
NosheiFrame.Configuration.RaidSuitable = false
NosheiFrame.Configuration.FrameType = "Frame"
NosheiFrame.Configuration.Width = nosheiFrameWidth
NosheiFrame.Configuration.Height = nosheiFrameHeight
--NosheiFrame.Configuration.Resizable = { 165, 40, 412, 100 }
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function NosheiFrame:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function NosheiFrame:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.8,
				color={r=0,g=0,b=0,a=0.6}, colorBinding="aggroColor",
			}, 
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, visibilityBinding="blockedOrRanged",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="RIGHT", element="frame", targetPoint="RIGHT", offsetX=-1 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", height=40, colorBinding="taggedCallingColor",
				texAddon=AddonId, texFile="img/textureCloud.png", 
				backgroundColor={r=0, g=0, b=0, a=1},
			},
			{
				-- Generic Element Configuration
				id="barResource", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="barHealth", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="resourcePercent", colorBinding="resourceColor",
				texAddon=AddonId, texFile="img/textureCloud.png",
				backgroundColor={r=0, g=0, b=0, a=1},
			},
			{
				-- Generic Element Configuration
				id="frameColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.2,
				attach = { 
					{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT" }
				},
				colorBinding="taggedCallingColor", 
				backgroundColor={r=0, g=0, b=0, a=1},
			},
			{
				-- Generic Element Configuration
				id="frameColor2", type="Frame", parent="frameBackdrop", layer=15, alpha=0.2,
				attach = { 
					{ point="TOPLEFT", element="barResource", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barResource", targetPoint="BOTTOMRIGHT" }
				},
				colorBinding="resourceColor", 
			},
			{
				id="barAbsorb", type="Bar", parent="frameBackdrop", layer=17,
				attach = {
					{ point="TOPLEFT", element="barHealth", targetPoint="CENTERLEFT" },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT" }
				},
				growthDirection="right", 
				binding="absorbPercent",
				color={r=0.08, g=0.91, b=1, a=0.8},
				backgroundColor={r=0.08, g=0.91, b=1, a=0}
			},
			{
				-- Generic Element Configuration
				id="labelHealth", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT", offsetX=0, offsetY=-40 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}/{healthMax}", default="", color={r=1,g=1,b=1,a=1}, fontSize=13
			},
			{
				-- Generic Element Configuration
				id="labelHealthR", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT", offsetX=0, offsetY=-20 }},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{healthPercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=13
			},
			{
				id="labelHealthAndAbsorbPercent", type="Label", parent="frame", layer=20,
				attach = {{ point="CENTERLEFT", element="barResource", targetPoint="CENTERLEFT", offsetX=5, offsetY=-20 }},
				visibilityBinding="absorb",
				text="{absorb}  {healthAbsorbPercent}%",
				color={r=0, g=0, b=0, a=1}, fontSize=13,
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERLEFT", element="barResource", targetPoint="CENTERLEFT", offsetX=5, offsetY=-40 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{name}", default="", color={r=1,g=1,b=1,a=1}, fontSize=14,
			},
			{
				-- Generic Element Configuration
				id="labelInfo", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERLEFT", element="barResource", targetPoint="CENTERLEFT", offsetX=5, offsetY=0 }},
				visibilityBinding="level",
				-- Type Specific Element Configuration
				text="{level} {callingText}", default="", color={r=1,g=1,b=1,a=1}, fontSize=14
			},
			{
				-- Generic Element Configuration
				id="labelRace", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERLEFT", element="labelInfo", targetPoint="CENTERRIGHT", offsetX=0, offsetY=0 }},
				visibilityBinding="raceName",
				-- Type Specific Element Configuration
				text="{raceName}", default="", color={r=1,g=1,b=1,a=1}, fontSize=14
			},
			{
				-- Generic Element Configuration
				id="labelNPCType", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="BOTTOMLEFT", element="frame", targetPoint="TOPLEFT", offsetX=5, offsetY=4 }},
				visibilityBinding="npcType",
				-- Type Specific Element Configuration
				text="{npcType}", default="", color={r=1,g=1,b=1,a=1}, fontSize=14
			},
			{
				-- Generic Element Configuration
				id="labelresource", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barResource", targetPoint="CENTERRIGHT" }},
				visibilityBinding="resource",
				-- Type Specific Element Configuration
				text="{resource}/{resourceMax}  {resourcePercent}%",
				default="", color={r=1,g=1,b=1,a=1}, fontSize=13
			},
			{
				-- Generic Element Configuration
				id="frameRole", type="Frame", parent="frame", layer=20, alpha=1,
				attach = {{ point="TOPRIGHT", element="frame", targetPoint="TOPLEFT",  offsetX=-1, offsetY=0 }},
				height=20, width = 20,
				color={r=0,g=0,b=0,a=0}, 
			},
			{
				id="imgRole", type="MediaSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frameRole", targetPoint="CENTER" }},
				width = 22, height = 22,
				nameBinding="role",
				names = 
					{ 
						["tank"] = "riftRaidRoleTank_Large", 
						["heal"] = "riftRaidRoleHeal_Large", 
						["dps"] = "riftRaidRoleDPS_Large", 
						["support"] = "riftRaidRoleSupport_Large" 
					}, 
			},
			{
				-- Generic Element Configuration
				id="frameCharges", type="Frame", parent="frame", layer=10, alpha=1,
				attach = {{ point="TOPLEFT", element="frameRole", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=1 }},
				visibilityBinding="planarMax",
				color={r=0.4236,g=0.4275,b=0.4236,a=1}, height=20, width = 20,
			},
			{
				-- Generic Element Configuration
				id="frameCharges2", type="Frame", parent="frame", layer=15, alpha=1,
				attach = {{ point="CENTER", element="frameCharges", targetPoint="CENTER" }},
				visibilityBinding="planarMax",
				color={r=0,g=0,b=0,a=1}, height=18, width = 18,
			},
			{
				-- Generic Element Configuration
				id="labelCharges", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTER", element="frameCharges", targetPoint="CENTER" }},
				-- Type Specific Element Configuration
				text="{planar}/{planarMax}", default="", color={r=1,g=1,b=1,a=1}, fontSize=11
			},
			{
				-- Generic Element Configuration
				id="frameVitality", type="Frame", parent="frame", layer=10, alpha=1,
				attach = {{ point="TOPLEFT", element="frameCharges", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=2 }},
				color={r=0,g=0,b=0,a=0}, height=20, width = 20,
			},
			{
				-- Generic Element Configuration
				id="imgVitality", type="ImageSet", parent="frame", layer=15, alpha=1.0,
				attach = {{ point="CENTER", element="frameVitality", targetPoint="CENTER" }},
				texAddon=AddonId, texFile="img/wtVitality.png", rows=10,cols=1,
				indexBinding="vitalityIndex", visibilityBinding="vitalityIndex",width=22,height=22,
			},
			{
				-- Generic Element Configuration
				id="txtVitality", type="Label", parent="frame", layer=20,
				attach = {{ point="CENTER", element="imgVitality", targetPoint="CENTER" }},
				text="{vitality}", visibilityBinding="vitalityIndex", fontSize=10,
			},
			{
				id="imgMark", type="MediaSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frame", targetPoint="TOPCENTER" }},
				width = 32, height = 32,
				nameBinding="mark",
				names = 
					{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
					},
				visibilityBinding="mark", alpha=1.0,
			},
			{
				id="imgPVP", type="MediaSet", parent="frame", layer=60, width=22, height=22,
				attach = {{ point="CENTER", element="frame", targetPoint="TOPRIGHT"}}, 
				nameBinding="pvpAlliance",
				names = 
				{
					["defiant"] = "FactionDefiant",
					["guardian"] = "FactionGuardian",
					["nightfall"] = "FactionNightfall",
					["oathsworn"] = "FactionOathsworn",
					["dominion"] = "FactionDominion",
				},
			},
			{
				-- Generic Element Configuration
				id="imgReady", type="ImageSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="frame", targetPoint="CENTER" }},
				-- Type Specific Element Configuration
				texAddon=AddonId, texFile="img/wtReady.png", nameBinding="readyStatus", cols=1, rows=2, 
				names = { ["ready"] = 0, ["notready"] = 1 }, defaultIndex = "hide",
			},
			{
			id="buffPanelBuffs", type="BuffPanel", parent="HorizontalBar", layer=30,
			attach = {{ point="TOPLEFT", element="frameBackdrop", targetPoint="BOTTOMLEFT", offsetX=0, offsetY=0 }},
			rows=5, cols=4, iconSize=24, iconSpacingHorizontal=1, iconSpacingVertical=12, borderThickness=1,
			acceptLowPriorityBuffs=false, acceptMediumPriorityBuffs=true, acceptHighPriorityBuffs=true, acceptCriticalPriorityBuffs=true,
			acceptLowPriorityDebuffs=false, acceptMediumPriorityDebuffs=false, acceptHighPriorityDebuffs=false, acceptCriticalPriorityDebuffs=false,
			growthDirection = "right_down", selfCast=false,
			timerSize=10, timerOffsetX=0, timerOffsetY=18,
			stackSize=20, stackOffsetX=0, stackOffsetY=0, stackBackgroundColor={r=0,g=0,b=0,a=0.3},
			borderColor={r=0,g=0,b=0,a=1},
			sweepOverlay=true,
		},
			
		}
	}
	
	for idx,element in ipairs(template.elements) do
		local showElement = true
		if options.excludeBuffs and element.type=="BuffPanel" then showElement = false end
		if not options.showAbsorb and (element.id=="barAbsorb" or element.id=="labelHealthAndAbsorbPercent") then 
			showElement = false 
		end
		if showElement then
			self:CreateElement(element)
		end 
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end


-- Local config options ---------------------------------------------------------
local nosheiFrameTTWidth = 160
local nosheiFrameTTHeight = 30
---------------------------------------------------------------------------------

-- Frame Configuration Options --------------------------------------------------
local NosheiFrameTT = WT.UnitFrame:Template("NosheiFrameTT")
NosheiFrameTT.Configuration.Name = "Replacement of nkThings Target of Target frame"
NosheiFrameTT.Configuration.RaidSuitable = false
NosheiFrameTT.Configuration.FrameType = "Frame"
NosheiFrameTT.Configuration.Width = nosheiFrameTTWidth
NosheiFrameTT.Configuration.Height = nosheiFrameTTHeight
---------------------------------------------------------------------------------

-- Override the buff filter to hide some buffs ----------------------------------
local buffPriorities = 
{
	["Track Wood"] = 0,
	["Track Ore"] = 0,
	["Track Plants"] = 0,
	["Track Fish"] = 0,
	["Rested"] = 0,
	["Prismatic Glory"] = 0,
	["Looking for Group Cooldown"] = 0,
}
function NosheiFrameTT:GetBuffPriority(buff)
	if not buff then return 2 end
	return buffPriorities[buff.name] or 2
end
---------------------------------------------------------------------------------

function NosheiFrameTT:Construct(options)

	local template =
	{
		elements = 
		{
			{
				-- Generic Element Configuration
				id="frameBackdrop", type="Frame", parent="frame", layer=0,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT" } 
				},
				visibilityBinding="id",
				-- Type Specific Element Configuration
				texAddon = AddonId, texFile = "img/wtMiniFrame_bg.png", alpha=0.8,
				color={r=0,g=0,b=0,a=0.6}, colorBinding="aggroColor",
			}, 
			{
				-- Generic Element Configuration
				id="frameBlocked", type="Frame", parent="frameBackdrop", layer=15, visibilityBinding="blockedOrRanged",
				color={r=0,g=0,b=0},alpha=0.6,
				attach = { 
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 } 
				},
			}, 
			{
				-- Generic Element Configuration
				id="barHealth", type="Bar", parent="frameBackdrop", layer=10,
				attach = {
					{ point="TOPLEFT", element="frame", targetPoint="TOPLEFT", offsetX=1, offsetY=1 },
					{ point="BOTTOMRIGHT", element="frame", targetPoint="BOTTOMRIGHT", offsetX=-1, offsetY=-1 },
				},
				-- visibilityBinding="id",
				growthDirection="right",
				-- Type Specific Element Configuration
				binding="healthPercent", colorBinding="taggedCallingColor",
				texAddon=AddonId, texFile="img/textureCloud.png", 
				backgroundColor={r=0, g=0, b=0, a=1},
			},
			{
				-- Generic Element Configuration
				id="frameColor", type="Frame", parent="frameBackdrop", layer=15, alpha=0.2,
				attach = { 
					{ point="TOPLEFT", element="barHealth", targetPoint="TOPLEFT" },
					{ point="BOTTOMRIGHT", element="barHealth", targetPoint="BOTTOMRIGHT" }
				},
				colorBinding="taggedCallingColor", 
				backgroundColor={r=0, g=0, b=0, a=1},
			},
			{
				-- Generic Element Configuration
				id="labelHealth", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERRIGHT", element="barHealth", targetPoint="CENTERRIGHT", offsetX=-2, offsetY=0}},
				visibilityBinding="health",
				-- Type Specific Element Configuration
				text="{health}/{healthPercent}%", default="", color={r=1,g=1,b=1,a=1}, fontSize=12
			},
			{
				-- Generic Element Configuration
				id="labelName", type="Label", parent="frame", layer=20, alpha=1,
				attach = {{ point="CENTERLEFT", element="barHealth", targetPoint="CENTERLEFT", offsetX=5, offsetY=0 }},
				visibilityBinding="name",
				-- Type Specific Element Configuration
				text="{nameMini}", default="", color={r=1,g=1,b=1,a=1}, fontSize=12,
			},
			{
				id="imgMark", type="MediaSet", parent="frame", layer=60,
				attach = {{ point="CENTER", element="labelName", targetPoint="CENTERRIGHT", offsetX=12, offsetY=0 }},
				width = 24, height = 24,
				nameBinding="mark",
				names = 
					{
						["1"] = "riftMark01",
						["2"] = "riftMark02",
						["3"] = "riftMark03",
						["4"] = "riftMark04",
						["5"] = "riftMark05",
						["6"] = "riftMark06",
						["7"] = "riftMark07",
						["8"] = "riftMark08",
						["9"] = "riftMark09",
						["10"] = "riftMark10",
						["11"] = "riftMark11",
						["12"] = "riftMark12",
						["13"] = "riftMark13",
						["14"] = "riftMark14",
						["15"] = "riftMark15",
						["16"] = "riftMark16",
						["17"] = "riftMark17",
					},
				visibilityBinding="mark", alpha=1.0,
			},
		}
	}
	
	for idx,element in ipairs(template.elements) do
		self:CreateElement(element)
	end
	
	self:SetSecureMode("restricted")
	self:SetMouseoverUnit(self.UnitSpec)
	self.Event.LeftClick = "target @" .. self.UnitSpec
	self.Event.RightClick = function() if self.UnitId then Command.Unit.Menu(self.UnitId) end end
end

WT.Unit.CreateVirtualProperty("vitalityDisplay", { "vitality" },
	function(unit)
		if type(unit.vitality) == "number" and unit.vitality < 100 then
			return unit.vitality
		else
			return nil
		end
	end)

WT.Unit.CreateVirtualProperty("npcType", { "level", "tier", "guaranteedLoot" },
	function(unit)
		if unit.tier and unit.level == "??" then
			return "Boss"
		elseif unit.tier then
			return "Elite"
		elseif unit.guaranteedLoot and unit.tier then
			return "Rare Elite"
		elseif unit.guaranteedLoot then
			return "Rare"
		else
			return nil
		end
	end)
	
WT.Unit.CreateVirtualProperty("taggedCallingColor", { "callingColor", "calling", "tagged", "relation" },
	function(unit)
		if unit.calling then
			return unit.callingColor
		elseif unit.tagged == "other" and unit.relation == "hostile" then
			return { r = 0.4, g = 0.4, b = 0.4, a = 1.0 }
		else
			return { r = 0, g = 0.7, b = 0, a = 1.0 }
		end
	end)

if not WT.Unit.VirtualProperties.vitalityIndex then
	WT.Unit.CreateVirtualProperty("vitalityIndex", {"vitality", "vitalityMax"}, 
		function(unit)
			local vitality = unit.vitality
			if not vitality then return nil end 
			if vitality > 90 then 
				return nil
				elseif vitality > 80 then return 0
				elseif vitality > 70 then return 1
				elseif vitality > 60 then return 2
				elseif vitality > 50 then return 3
				elseif vitality > 40 then return 4
				elseif vitality > 30 then return 5
				elseif vitality > 20 then return 6
				elseif vitality > 10 then return 7
				elseif vitality > 0 then return 8
				else return 9
			end
		end)
end