--[[
File: Config.lua
Author: Zormus
--]]

local Interface = Library.LibZormus.Interface
local addon, Roles = ...

Roles.config = Interface.Create("Context", "Config")
local config = Roles.config
local roles

local function SetEquipSet(button, set)
	local equip = config.equip[button]
	if set then
		equip.labelItems:SetAlpha(1)
		equip.items:SetEnabled(true)
	else
		equip.labelItems:SetAlpha(0.5)
		equip.items:SetEnabled(false)
		if equip.items:GetText() ~= "" then
			equip.items:SetText("")
		end
	end
end

local function TrimLeadingSlash(text)
	local char = string.sub(text, 1, 1)
	while char == "/" do
		text = string.sub(text, 2)
		char = string.sub(text, 1, 1)
	end
	return text
end

local function SetSelectedRole(update)
	local newRole = roles[config.role:GetValue()]
	config.equip.Left.select:Close()
	config.equip.Right.select:Close()
	config.equip.Left.items:SetKeyFocus(false)
	config.equip.Right.items:SetKeyFocus(false)
	config.custom:SetKeyFocus(false)
	if update then
		config.selectedRole:SetMenu(config.menu:GetChecked())
		local custom = {}
		for i, line in ipairs(config.custom:GetLines()) do
			line = TrimLeadingSlash(line)
			if string.len(line) > 0 then
				table.insert(custom, line)
			end
		end
		config.selectedRole:SetCustom(custom)
	end
	config.menu:SetChecked(newRole:GetMenu())
	config.custom:SetLines(newRole:GetCustom())
	for _, button in ipairs{"Left", "Right"} do
		if update then
			config.selectedRole:SetEquip(button, config.equip[button].select:GetValue())
			config.selectedRole:SetItems(button, config.equip[button].items:GetLines())
		end
		local set = newRole:GetEquip(button)
		config.equip[button].select:SetValue(set)
		SetEquipSet(button, set)
		config.equip[button].items:SetLines(newRole:GetItems(button))
	end
	config.selectedRole = newRole
end

local InitializeTab = {}
local LeaveTab = {}

local function CreateLabel(text, parent)
	local label = Interface.Create("Text", "Label", parent)
	label:SetText(text)
	label:SetFontSize(13)
	label:SetLayer(-1)
	return label
end

local function CreateHeaderLabel(text, parent)
	local label = Interface.Create("Text", "HeaderLabel", parent)
	label:SetText(text)
	label:SetFontSize(14)
	label:SetFontColor(0.96, 0.87, 0.74, 1)
	label:SetEffectGlow({colorR = 0.54, colorG = 0.42, colorB = 0.22, strength = 1})
	label:SetLayer(-1)
	return label
end

local function CreateDivider(parent)
	local divider = Interface.Create("Frame", "DividerTop", parent)
	divider:SetWidth(parent:GetWidth() - 4)
	divider:SetHeight(2)
	divider:SetPoint("CENTERX", parent, "CENTERX")
	divider:SetBackgroundColor(0.3, 0.3, 0.3, 1)
	return divider
end

function InitializeTab.ROLES(tab)
	config.role = Interface.Create("RiftSelect", "RoleSelect", tab)
	config.role:SetPoint("TOPLEFT", tab, "TOPLEFT", 62, 0)
	config.role:SetWidth(240)
	local items = {}
	for index, role in ipairs(roles) do
		items[index] = {value = index, label = string.format("%i - %s", index, role:GetName())}
	end
	config.role:SetItems(items)
	config.role.OnChange = function() SetSelectedRole(true) end

	config.labelRole = CreateLabel("Role:", tab)
	config.labelRole:SetPoint(1, 0.5, config.role, 0, 0.5, -4, 1)
	config.menu = Interface.Create("RiftCheckbox", "InMenu", tab)
	config.menu:SetPoint(0, 0.5, config.role, 1, 0.5, 28, 0)
	config.menu:SetLabel("Include role in menu")

	config.dividerRole = CreateDivider(tab)
	config.dividerRole:SetPoint("TOP", config.role, "BOTTOM", nil, 11)
	
	config.equip = {}
	items = {{value = nil, label = "None"}}
	for index = 1, 9 do
		items[index + 1] = {value = index, label = string.format("Set %i", index)}
	end
	for _, button in ipairs{"Left", "Right"} do
		config.equip[button] = Interface.Create("Frame", "Equip" .. button, tab)
		local equip = config.equip[button]
		equip:SetPoint(string.upper(button), tab, string.upper(button))
		equip:SetPoint("TOP", config.role, "BOTTOM", nil, 22)
		equip:SetWidth(241)
		
		equip.labelClick = CreateHeaderLabel(string.format("%s Click", button), equip)
		equip.labelClick:SetPoint("TOPLEFT", equip, "TOPLEFT")
		equip.labelLoad = CreateLabel("Load equipment:", equip)
		equip.labelLoad:SetPoint("TOPLEFT", equip.labelClick, "BOTTOMLEFT", 0, 4)
		
		equip.select = Interface.Create("RiftSelect", "EquipSelect" .. button, equip)
		equip.select:SetItems(items)
		equip.select:SetPoint(0, 0.5, equip.labelLoad, 1, 0.5, 4, -1)
		equip.select:SetPoint("RIGHT", equip, "RIGHT")
		function equip.select.OnChange(self)
			SetEquipSet(button, self:GetValue())
		end
		
		equip.labelItems = CreateLabel("Replacement items:", equip)
		equip.labelItems:SetPoint("TOPLEFT", equip.labelLoad, "BOTTOMLEFT", 0, 6)
		equip.items = Interface.Create("RiftTextfield", "EquipItems", equip)
		equip.items:SetLayer(-1)
		equip.items:SetPoint("TOPLEFT", equip.labelItems, "BOTTOMLEFT", 0, 2)
		equip.items:SetPoint("RIGHT", equip, "RIGHT")
		equip.items:SetHeight(66)
		equip.items:SetMaximumLines(4)
		
		equip:SetPoint("BOTTOM", equip.items, "BOTTOM")
	end
	config.labelCustom = CreateLabel("Additional macro text:", tab)
	config.labelCustom:SetPoint("TOPLEFT", config.equip.Left, "BOTTOMLEFT", 0, 4)
	config.custom = Interface.Create("RiftTextfield", "CustomMacro", tab)
	config.custom:SetLayer(-1)
	config.custom:SetPoint("TOPLEFT", config.labelCustom, "BOTTOMLEFT", 0, 2)
	config.custom:SetPoint("RIGHT", tab, "RIGHT")
	config.custom:SetHeight(92)
	config.custom:SetMaximumLines(6)
end

local function UpdateSettings()
	if config.settings then
		_config.sort = config.sort:GetValue()
		_config.extend = config.extend:GetValue()
	end
end

function Roles.UpdateLockButton()
	if config.lock then
		if _config.lock then
			config.lock:SetText("UNLOCK")
		else
			config.lock:SetText("LOCK")
		end
	end
end

local function ClickLock()
	Roles.SlashCommand(nil, "lock")
end

local function ClickReset()
	Roles.SlashCommand(nil, "reset")
end

function InitializeTab.SETTINGS(tab)
	config.labelSort = CreateLabel("Menu order:", tab)
	config.labelSort:SetPoint("TOPRIGHT", tab, "TOPLEFT", 144, 56)
	
	config.sort = Interface.Create("RiftSelect", "Sort", tab)
	local items =
	{
		{value = "Slot", label = "Sort roles by slot number"},
		{value = "Name", label = "Sort roles alphabetically by name"},
		{value = "Type", label = "Sore roles by type, then by name"}
	}
	config.sort:SetItems(items)
	config.sort:SetWidth(280)
	config.sort:SetPoint(0, 0.5, config.labelSort, 1, 0.5, 8, -1)
	config.sort:SetLayer(2)
	config.sort:SetValue(_config.sort)
	
	config.labelExtend = CreateLabel("Menu direction:", tab)
	config.labelExtend:SetPoint("TOPRIGHT", config.labelSort, "BOTTOMRIGHT", 0, 40)
	
	config.extend = Interface.Create("RiftSelect", "Extend", tab)
	items =
	{
		{value = "Down", label = "Extend menu downward"},
		{value = "Up", label = "Extend menu upward"}
	}
	config.extend:SetItems(items)
	config.extend:SetWidth(280)
	config.extend:SetPoint(0, 0.5, config.labelExtend, 1, 0.5, 8, -1)
	config.extend:SetValue(_config.extend)
	
	config.labelWidth = CreateLabel("Menu width:", tab)
	config.labelWidth:SetPoint("TOPRIGHT", config.labelExtend, "BOTTOMRIGHT", 0, 40)
	
	config.width = Interface.Create("RiftSlider", "Width", tab)
	config.width:SetPoint(0, 0.5, config.labelWidth, 1, 0.5, 10, 8)
	config.width:SetWidth(276)
	config.width:SetRange(Roles.minWidth, Roles.maxWidth)
	config.width:SetPosition(_config.width)
	local function SliderChange(self)
		_config.width = self:GetPosition()
		Roles.menu:SetWidth(_config.width)
	end	
	config.width:EventAttach(Event.UI.Slider.Change, SliderChange, "SliderChange")

	config.labelLocation = CreateLabel("Menu location:", tab)
	config.labelLocation:SetPoint("TOPRIGHT", config.labelWidth, "BOTTOMRIGHT", 0, 40)
	
	config.lock = Interface.Create("RiftButton", "Lock", tab)
	config.lock:SetPoint(0, 0.5, config.labelLocation, 1, 0.5, 10, 0)
	Roles.UpdateLockButton()
	config.lock:EventAttach(Event.UI.Input.Mouse.Left.Click, ClickLock, "ClickLock")
	
	config.reset = Interface.Create("RiftButton", "Reset", tab)
	config.reset:SetPoint("TOPLEFT", config.lock, "TOPRIGHT", 6, 0)
	config.reset:SetText("RESET")
	config.reset:EventAttach(Event.UI.Input.Mouse.Left.Click, ClickReset, "ClickReset")
	
	config.settings = true
end

local sections =
{
	{header = "Role Icons", text = "To fill in the icon for a role, simply switch to the role and the addon will automatically update and save the information."},
	{header = "Role Names", text = "You can edit your role names inside the soul tree window. Updated names will appear the next time you open the role menu."},
	{header = "Equipment Sets", text = "To save an equipment set, use the \"/saveequip #\" command, where # is a number between 1 and 9. This set can then be used by the addon. Replacement items allow you to reuse an existing set with minor modifications."},
	{header = "Role Menu", text = "Left-click the menu button to open/close the menu, middle-click to open the configuration window, and right-click to move the menu when unlocked."}
}

function InitializeTab.HELP(tab)
	for index, section in ipairs(sections) do
		section.headerLabel = CreateHeaderLabel(section.header, tab)
		if index == 1 then
			section.headerLabel:SetPoint("TOPLEFT", tab, "TOPLEFT")
		else
			section.headerLabel:SetPoint("TOPLEFT", sections[index - 1].textLabel, "BOTTOMLEFT", 0, 6)
		end
		section.textLabel = CreateLabel(section.text, tab)
		section.textLabel:SetPoint("TOPLEFT", section.headerLabel, "BOTTOMLEFT")
		section.textLabel:SetPoint("RIGHT", tab, "RIGHT")
		section.textLabel:SetWordwrap(true)
	end
	config.labelVersion = CreateLabel(string.format("Version: %s", addon.toc.Version), tab)
	config.labelVersion:SetPoint("BOTTOMLEFT", tab, "BOTTOMLEFT", 0, 0)
	config.labelAuthor = CreateLabel(string.format("Author: %s", addon.toc.Author), tab)
	config.labelAuthor:SetPoint("BOTTOMRIGHT", tab, "BOTTOMRIGHT", 0, 0)
end

function LeaveTab.ROLES(tab)
	config.role:Close()
	config.equip.Left.select:Close()
	config.equip.Right.select:Close()
	config.equip.Left.items:SetKeyFocus(false)
	config.equip.Right.items:SetKeyFocus(false)
	config.custom:SetKeyFocus(false)
end

function LeaveTab.SETTINGS(tab)
	config.sort:Close()
	config.extend:Close()
end

function LeaveTab.HELP(tab)
end

local function OnInitializeTab(tab, name)
	InitializeTab[name](tab)
end

local function OnLeaveTab(tab, name)
	LeaveTab[name](tab)
end

local function InitializeDialog(self)
	self:SetTitle(string.upper(addon.identifier))
	self:SetSize(592, 516)

	self.tabview = Interface.Create("RiftTabview", "Tabview", self)
	self.tabview.OnInitializeTab = OnInitializeTab
	self.tabview.OnLeaveTab = OnLeaveTab
	self.tabview:SetPoint("TOPLEFT", self, "TOPLEFT", 30, 70)
	self.tabview:SetPoint("TOPRIGHT", self, "TOPRIGHT", -30, 70)
	self.tabview:SetPoint("BOTTOM", self, "BOTTOM", nil, -62)
	local tabs = {"ROLES", "SETTINGS", "HELP"}
	for _, tab in ipairs(tabs) do
		self.tabview:AddTab(tab)
	end
	
	self.closeButton = Interface.Create("RiftButton", "Close", self)
	self.closeButton:SetPoint(0.5, 0, self.tabview, 0.5, 1, 0, 4)
	self.closeButton:SetText("CLOSE")
	self.closeButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self) self:GetParent():Close() end, "Close")
	self.closeButton:SetLayer(-1)
	function self:OnClose()
		LeaveTab[self.tabview:GetActiveTab()]()
		SetSelectedRole(true)
		UpdateSettings()
		if self:GetParent().OnClose then
			self:GetParent().OnClose()
		end
	end
end

function config:Show()
	if not self.dialog then
		roles = Roles.roles
		self.dialog = Interface.Create("RiftWindow", "Dialog", self)
		self.dialog:SetVisible(false)
		InitializeDialog(self.dialog)
		self.dialog:Center()
	end
	self.role:SetValue(Roles.GetCurrentRole():GetSlot())
	self.selectedRole = Roles.GetCurrentRole()
	SetSelectedRole()
	self.dialog:SetVisible(true)
end

function config:IsOpen()
	return self.dialog and self.dialog:GetVisible()
end
