--[[
File: Interface/Canvas.lua
Author: Zormus
--]]

local Interface = Library[(...).identifier].Interface
Interface.Canvas = setmetatable({}, Interface.Frame)
local Canvas = Interface.Canvas
Canvas.__index = Canvas

Canvas.type = "Frame"

function Canvas:Initialize()
	self.margins = {LEFT = 0, TOP = 0, RIGHT = 0, BOTTOM = 0}
end

local function UpdateLayout(self)
	local centerWidth = self:GetWidth() - self.margins.LEFT - self.margins.RIGHT
	local centerHeight = self:GetHeight() - self.margins.TOP - self.margins.BOTTOM
	local horizontalScale = centerWidth / (self.panels.TOPLEFT.image:GetWidth() - self.margins.LEFT - self.margins.RIGHT)
	local verticalScale = centerHeight / (self.panels.TOPLEFT.image:GetHeight() - self.margins.TOP - self.margins.BOTTOM)
	local offset =
	{
		LEFT = -self.margins.LEFT * horizontalScale,
		TOP = -self.margins.TOP * verticalScale,
		RIGHT = self.margins.RIGHT * horizontalScale,
		BOTTOM = self.margins.BOTTOM * verticalScale
	}
	for _, x in pairs({"LEFT", "CENTER", "RIGHT"}) do
		for _, y in pairs({"TOP", "CENTER", "BOTTOM"}) do
			local name = y .. x
			if x == "CENTER" and y == "CENTER" then
				self.panels[name]:SetSize(centerWidth, centerHeight)
				for i, j in pairs({LEFT = "TOP", RIGHT = "BOTTOM"}) do
					self.panels[name].image:SetPoint(j .. i, self.panels[name], j .. i, offset[i], offset[j])
				end
			elseif x == "CENTER" then
				self.panels[name]:SetSize(centerWidth, self.margins[y])
				for _, i in pairs({"LEFT", "RIGHT"}) do
					self.panels[name].image:SetPoint(y .. i, self.panels[name], y .. i, offset[i], 0)
				end
			elseif y == "CENTER" then
				self.panels[name]:SetSize(self.margins[x], centerHeight)
				for _, j in pairs({"TOP", "BOTTOM"}) do
					self.panels[name].image:SetPoint(j .. x, self.panels[name], j .. x, 0, offset[j])
				end
			else
				self.panels[name]:SetSize(self.margins[x], self.margins[y])
			end
		end
	end
end

local function CreatePanels(self)
	self.panels = {}
	for _, x in pairs({"LEFT", "CENTER", "RIGHT"}) do
		for _, y in pairs({"TOP", "CENTER", "BOTTOM"}) do
			local name = y .. x
			self.panels[name] = Interface.Create("Mask", name, self)
			self.panels[name]:SetLayer(-1)
			self.panels[name].image = Interface.Create("Image", "Image", self.panels[name])
			if x == "CENTER" and y == "CENTER" then
				self.panels[name]:SetPoint("TOPLEFT", self.panels["TOPLEFT"], "BOTTOMRIGHT")
			elseif x == "CENTER" then
				self.panels[name]:SetPoint(y .. "LEFT", self.panels[y .. "LEFT"], y .. "RIGHT")
			elseif y == "CENTER" then
				self.panels[name]:SetPoint("TOP" .. x, self.panels["TOP" .. x], "BOTTOM" .. x)
			else
				self.panels[name]:SetPoint(name, self, name)
				self.panels[name].image:SetPoint(name, self.panels[name], name)
			end
		end
	end
	self:EventAttach(Event.UI.Layout.Size, UpdateLayout, "SizeChanged")
end

function Canvas:SetBackgroundImage(image, source)
	local width, height = 0, 0
	if self.panels then
		width = self.panels.TOPLEFT.image:GetWidth()
		height = self.panels.TOPLEFT.image:GetHeight()
	else
		CreatePanels(self)
	end
	for _, panel in pairs(self.panels) do
		panel.image:SetImage(image, source)
	end
	if self.panels.TOPLEFT.image:GetWidth() ~= width or self.panels.TOPLEFT.image:GetHeight() ~= height then
		UpdateLayout(self)
	end
end

function Canvas:SetMargins(left, top, right, bottom)
	self.margins.LEFT = left
	self.margins.TOP = top
	self.margins.RIGHT = right
	self.margins.BOTTOM = bottom
	if self.panels then
		UpdateLayout(self)
	end
end
