--[[
File: Interface/RiftSelect.lua
Author: Zormus
--]]

local Interface = Library[(...).identifier].Interface
Interface.RiftSelect = setmetatable({}, Interface.Canvas)
local RiftSelect = Interface.RiftSelect
RiftSelect.__index = RiftSelect

function RiftSelect:Initialize()
	Interface.Canvas.Initialize(self)
	self:SetHeight(32)
	self:SetMargins(12, 10, 30, 10)
	self:SetBackgroundImage("drop_down_(normal)02.png.dds")
	self:EnableHighlight(true)
	self:EnableClick(true, "Left")
	self.text = Interface.Create("Text", "Text", self)
	self.text:SetPoint("TOPLEFT", self, "TOPLEFT", 9, 7)
	self.text:SetPoint("RIGHT", self, "RIGHT", -29, nil)
	self.text:SetFontSize(13)
	self:SetLayer(1)
end

function RiftSelect:OnHighlight(highlight)
	if not (self.menu and self.menu:GetVisible()) then
		if highlight then
			self:SetBackgroundImage("drop_down_(over)02.png.dds")
		else
			self:SetBackgroundImage("drop_down_(normal)02.png.dds")
		end
	end
end

local function CreateMenu(self)
	self.menu = Interface.Create("Canvas", "Menu", self)
	self.menu:SetVisible(false)
	self.menu:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
	self.menu:SetPoint("TOPRIGHT", self, "BOTTOMRIGHT")
	self.menu:SetMargins(10, 10, 10, 10)
	self.menu:SetBackgroundImage("dropdown_list.png.dds")
	self.menu.items = {}
end

local function SetSelectedItem(self, item)
	self.value = item.value
	self.text:SetText(item:GetText())
end

function RiftSelect:SetItems(items)
	if not self.menu then
		CreateMenu(self)
	end
	local itemHeight = 22
	self.menu.count = #items
	self.menu:SetHeight(self.menu.count * itemHeight + 14)
	local function OnHighlight(self, highlight)
		if highlight then
			self:SetBackgroundColor(0.1, 0.1, 0.1, 1)
		else
			self:SetBackgroundColor(0.1, 0.1, 0.1, 0)
		end
	end
	local function OnLeftClick(self)
		local select = self:GetParent():GetParent()
		if select.value ~= self.value then
			SetSelectedItem(select, self)
			if select.OnChange then
				select:OnChange()
			end
		end
		select:Close()
	end
	for index, item in ipairs(items) do
		if not self.menu.items[index] then
			self.menu.items[index] = Interface.Create("Text", "Item" .. index, self.menu)
			self.menu.items[index]:SetPoint("TOPLEFT", self.menu, "TOPLEFT", 9, (index - 1) * itemHeight + 4)
			self.menu.items[index]:SetPoint("TOPRIGHT", self.menu, "TOPRIGHT", -9, (index - 1) * itemHeight + 4)
			self.menu.items[index]:SetHeight(itemHeight)
			self.menu.items[index]:SetFontSize(13)
			self.menu.items[index]:EnableHighlight(true)
			self.menu.items[index].OnHighlight = OnHighlight
			self.menu.items[index]:EnableClick(true, "Left")
			self.menu.items[index].OnLeftClick = OnLeftClick
		end
		self.menu.items[index].value = item.value
		self.menu.items[index]:SetText(item.label)
		self.menu.items[index]:SetVisible(true)
	end
	for index = self.menu.count + 1, #self.menu.items do
		self.menu.items[index]:SetVisible(false)
	end
	SetSelectedItem(self, self.menu.items[1])
end

function RiftSelect:OnLeftClick()
	if self.menu then
		self.menu:SetVisible(not self.menu:GetVisible())
		if self.menu:GetVisible() then
			self:SetBackgroundImage("drop_down_(click)02.png.dds")
		else
			self:SetBackgroundImage("drop_down_(over)02.png.dds")
		end
	end
end

function RiftSelect:GetValue()
	return self.value
end

function RiftSelect:SetValue(value)
	if self.menu then
		for index = 1, self.menu.count do
			if self.menu.items[index].value == value then
				SetSelectedItem(self, self.menu.items[index])
				return
			end
		end
	end
end

function RiftSelect:Close()
	if self.menu then
		self.menu:SetVisible(false)
	end
end
