local addon, private = ...

um_OutfitSave = {}
um_Update = nil

UM_GLOBAL = {
	AddonData = Inspect.Addon.Detail("UpgradeMe"),
	Calling = nil,
	StatData = nil,
	OutfitData = nil,
	Outfits = nil,
	UI = nil,
	Config = nil,
	Tooltip = nil,
	x_tip = 0,
	y_tip = 0,
	debug = true,
	flgTooltip = false,
	OutfitDirty = true,
}

UM_GLOBAL.UI = UI.CreateContext("UpgradeMe")
UM_GLOBAL.CallingUpdates = "~none~"

function UM_GLOBAL.Initialize(addon)
	if addon == "UpgradeMe" then
		print(string.format("UpgradeMe v.%s loaded.", UM_GLOBAL.AddonData.toc.Version))
		UM_GLOBAL.StatData = GetStatData()
		UM_GLOBAL.OutfitData = GetOutfitData()
		UM_GLOBAL.Config = GetConfig(UM_GLOBAL.UI, UM_GLOBAL.StatData)
		UM_GLOBAL.Config.Event.Close = function() UM_GLOBAL.Config:Hide() end
		UM_GLOBAL.Tooltip = umTooltip(UM_GLOBAL.UI, UM_GLOBAL.OutfitData:OutfitCount())
	end
end

function UM_GLOBAL.Update()
	local _player = Inspect.Unit.Detail(Inspect.Unit.Lookup("player"))
	if not _player then return end
	if UM_GLOBAL.Calling and not UM_GLOBAL.OutfitDirty then return end
	if not Inspect.Unit.Lookup("player") then return end

	if not UM_GLOBAL.Calling then
		UM_GLOBAL.Calling = _player.calling

		if UM_GLOBAL.CallingUpdates:find(UM_GLOBAL.Calling) and um_Update ~= tostring(UM_GLOBAL.AddonData.toc.Version) then
			--UM_GLOBAL.Config:Show(1)
		end
		um_Update = tostring(UM_GLOBAL.AddonData.toc.Version)
	end

	if UM_GLOBAL.OutfitDirty then UM_GLOBAL.ValidateOutfit() end
end

function UM_GLOBAL.scrapeTooltip(_ttype, _titem, _tbuff)
	if _ttype ~= "item" and _ttype ~= "itemtype" then return end
	local _tslot = nil
	local _tscore = nil
	local _eslot = nil
	local _eitem = nil
	local _escore = 0
	local _escore2 = 0
	local _tips = nil
	local _title = nil
	local _umtip = ""
	local _umicon = ""
	local _detail = Inspect.Item.Detail(_titem)

	if not _detail then return end
	if not UM_GLOBAL.Outfits then return end

	if _detail.requiredCalling then
		if not string.find(_detail.requiredCalling, UM_GLOBAL.Calling) then return end
	end

	_tslot = UM_GLOBAL.getItemSlotName(_detail.category)
	if not _tslot then return end

	UM_GLOBAL.Tooltip:Hide()

	UM_GLOBAL.Tooltip:ClearTips()
	for oname, outfit in pairs(UM_GLOBAL.Outfits) do
		_tscore = 0
		_escore = 0
		_escore2 = 0

		_tscore = UM_GLOBAL.getItemScore(outfit, _titem)

		_umtip = "Outfit: "..oname

		if _tslot == "twohand" then
			if outfit["OutfitItems"]["handmain"] and outfit["OutfitItems"]["handmain"]["ID"] then _escore = UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"]["handmain"]["ID"]) end
			if outfit["OutfitItems"]["handoff"] and outfit["OutfitItems"]["handoff"]["ID"] then _escore = _escore + UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"]["handoff"]["ID"]) end
			if outfit["OutfitItems"]["twohand"] and outfit["OutfitItems"]["twhohand"]["ID"] then _escore =  _escore + UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"]["twohand"]["ID"]) end
		elseif _tslot == "ring" then
			if outfit["OutfitItems"]["ring1"] and outfit["OutfitItems"]["ring1"]["ID"] then _escore = UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"]["ring1"]["ID"]) end
			if outfit["OutfitItems"]["ring2"] and outfit["OutfitItems"]["ring2"]["ID"] then _escore2 = UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"]["ring2"]["ID"]) end
		else
			if outfit["OutfitItems"][_tslot..""] and outfit["OutfitItems"][_tslot..""]["ID"] then _escore = UM_GLOBAL.getItemScore(outfit, outfit["OutfitItems"][_tslot..""]["ID"]) end
		end

		if (_tscore > _escore) or (_tscore > _escore2 and _escore2 ~= 0) then
			_umtip = _umtip.." - Upgrade"
			_umicon = "Up"
			if _tslot ~= "ring" then
				_umtip = _umtip.." (".._tscore.."/".._escore..") +"..(_tscore - _escore)
			else
				if (_tscore > _escore) then _umtip = _umtip.." - Ring 1 (".._tscore.."/".._escore..") +"..(_tscore-_escore) end
				if (_tscore > _escore2) then _umtip = _umtip.." - Ring 2 (".._tscore.."/".._escore2..") +"..(_tscore-_escore2) end
			end
		elseif (_tscore < _escore) or (_tscore < _escore2 and _escore2 ~= 0) then
			_umtip = _umtip.." - Downgrade (".._tscore.."/".._escore..") -"..(_escore-_tscore)
			_umicon = "Down"
		else
			_umtip = _umtip.." - Equal"
			_umicon = "Side"
		end

		--table.insert(_tips, {_umtip, _umicon})
		UM_GLOBAL.Tooltip:AddTip(_umtip, _umicon)
	end

	UM_GLOBAL.Tooltip:Show()

	UM_GLOBAL.y_tip = Inspect.Mouse()["y"]
	UM_GLOBAL.x_tip = Inspect.Mouse()["x"]
end

function UM_GLOBAL.RoundValue(num, idp)
  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult
end

function UM_GLOBAL.getItemScore(_ofit, _itm)
	local _score = 0
	local _detail = nil

	_detail = Inspect.Item.Detail(_itm)

	if _detail and _detail.stats then
		_score = 0
		for statname, statrec in pairs(_ofit["OutfitWeights"]) do
			if _detail.stats[statname] then
				_score = _score + (_detail.stats[statname..""] * statrec["StatWeight1"])
			end
			if _detail.damageMax and statname == "WeaponDPS" then
				_score = _score + ((((_detail.damageMin + _detail.damageMax)/2)/_detail.damageDelay) * statrec["StatWeight1"])
			end
		end
	end

	return UM_GLOBAL.RoundValue(_score, 0)
end

function UM_GLOBAL.getItemSlotName(_itm)
	if not _itm then return nil end

	local _slot = nil
	if string.find(_itm, "head") or (_itm == "seqp.hlm") then
		_slot =  "helmet"
	elseif string.find(_itm, "cape") or (_itm == "seqp.cpe") then
		_slot =  "cape"
	elseif string.find(_itm, "shoulders") or (_itm == "seqp.shl") then
		_slot =  "shoulders"
	elseif string.find(_itm, "chest") or (_itm == "seqp.chs") then
		_slot =  "chest"
	elseif string.find(_itm, "hands") or (_itm == "seqp.glv") then
		_slot =  "gloves"
	elseif string.find(_itm, "waist") or (_itm =="seqp.blt") then
		_slot =  "belt"
	elseif string.find(_itm, "legs") or (_itm =="seqp.lgs") then
		_slot =  "legs"
	elseif string.find(_itm, "feet") or (_itm =="seqp.fet") then
		_slot =  "feet"
	elseif string.find(_itm, "onehand") or (_itm =="seqp.hmn") then
		_slot =  "handmain"
	elseif string.find(_itm, "twohand") then
		_slot =  "twohand"
	elseif string.find(_itm, "offhand") or string.find(_itm, "totem") or string.find(_itm, "shield") or (_itm =="seqp.hof") then
		_slot =  "handoff"
	elseif string.find(_itm, "ranged") or (_itm =="seqp.rng") then
		_slot =  "ranged"
	elseif string.find(_itm, "neck") or (_itm =="seqp.nck") then
		_slot =  "neck"
	elseif string.find(_itm, "trinket") or (_itm =="seqp.tkt") then
		_slot =  "trinket"
	elseif string.find(_itm, "ring") or string.find(_itm, "seqp.rn") then
		_slot =  "ring"
	elseif string.find(_itm, "synergy") or (_itm =="seqp.syn") then
		_slot =  "synergy"
	elseif string.find(_itm, "focus") or (_itm =="seqp.fcs") then
		_slot =  "focus"
	elseif string.find(_itm, "seal") or (_itm =="seqp.sel") then
		_slot =  "seal"
	end

	return _slot
end

local function HideTooltip(_x, _y)
	if not UM_GLOBAL.Tooltip then return end
	if not UM_GLOBAL.Tooltip:GetVisible() then return end

	if math.abs(_x - UM_GLOBAL.x_tip) > 30 or math.abs(_y - UM_GLOBAL.y_tip) > 30 then
		UM_GLOBAL.Tooltip:Hide()
	end
end

function UM_GLOBAL.SetOutfitDirty(_updates)
	for _slot, _id in pairs(_updates) do
		if string.find(_slot, "seqp.") then UM_GLOBAL.OutfitDirty = true end
	end
end

function UM_GLOBAL.ValidateOutfit()
	local _found = true
	UM_GLOBAL.flgTooltip = true

	for _oname, _outfit in pairs(UM_GLOBAL.Outfits) do
		_found = true
		for _slot, _item in pairs(_outfit["OutfitItems"]) do
			if not UM_GLOBAL.GearInSet(_item["ID"], _slot) then
				_found = false
			end
		end
		if _found then UM_GLOBAL.flgTooltip = false end
	end
	UM_GLOBAL.Tooltip:Hide()
	UM_GLOBAL.Tooltip:Show()
	UM_GLOBAL.OutfitDirty = false
end

function UM_GLOBAL.GearInSet(_id, _slot)
	local _ret = false
	local _idetail = Inspect.Item.Detail(Utility.Item.Slot.Equipment(_slot))

	if _idetail then
		if _id == _idetail.id then _ret = true end
	elseif not _id then
		_ret = true
	end
	return _ret
end


function UM_GLOBAL.LoadVariables(addon)
	if addon ~= "UpgradeMe" then return end
	UM_GLOBAL.Outfits = {}

	if #um_OutfitSave == 0 then return end
	for oNum, outfit in pairs(um_OutfitSave) do
		UM_GLOBAL.Outfits[outfit[1]..""] = {["Name"]=outfit[1], ["Description"]=outfit[2], ["OutfitItems"]={}, ["OutfitWeights"]={}}

		for oItem, item in pairs(outfit[3]) do
			if item[3] then
				UM_GLOBAL.Outfits[outfit[1]..""]["OutfitItems"][item[1]..""] = {["Slot"]=item[1], ["Name"]=item[2], ["ID"]=item[3]}
			end
		end

		for oWeight, weight in pairs(outfit[4]) do
			UM_GLOBAL.Outfits[outfit[1]..""]["OutfitWeights"][weight[1]..""] = {["StatName"]=weight[1], ["StatWeight1"]=weight[2], ["StatCap"]=weight[3], ["StatWeight2"]=weight[4]}
		end
	end
end

function UM_GLOBAL.SaveVariables(addon)
	if addon ~= "UpgradeMe" then return end
	um_OutfitSave  = {}

	for oName, outfit in pairs(UM_GLOBAL.Outfits) do
		table.insert(um_OutfitSave, {outfit["Name"], outfit["Description"], {}, {}})

		for oItem, item in pairs(outfit["OutfitItems"]) do
			table.insert(um_OutfitSave[#um_OutfitSave][3], {item["Slot"], item["Name"], item["ID"]})
		end

		for oWeight, weight in pairs(outfit["OutfitWeights"]) do
			table.insert(um_OutfitSave[#um_OutfitSave][4], {weight["StatName"], weight["StatWeight1"], weight["StatCap"], weight["StatWeight2"]})
		end
	end
end

function UM_GLOBAL.trim(_s)
	return _s:find'^%s*$' and '' or _s:match'^%s*(.*%S)'
end

function UM_GLOBAL.SlashHandler(args)
	args = UM_GLOBAL.trim(args)
	if not args or args == "" then
		UM_GLOBAL.Config:Show(2)
		return
	end

	UM_GLOBAL.Config:SaveOutfit(args)
end

table.insert(Event.Addon.Load.End, {UM_GLOBAL.Initialize, "UpgradeMe", "UpgradeMe_Init"})
table.insert(Command.Slash.Register("upme"), {UM_GLOBAL.SlashHandler, "UpgradeMe", "Slash Command"})
table.insert(Event.Tooltip, {UM_GLOBAL.scrapeTooltip, "UpgradeMe", "UpgradeScraper"})
table.insert(Event.Mouse.Move, {HideTooltip, "UpgradeMe", "UpgradeHider"})
table.insert(Event.Addon.SavedVariables.Save.Begin, {UM_GLOBAL.SaveVariables, "UpgradeMe", "UM_GLOBAL.SaveVar"})
table.insert(Event.Addon.SavedVariables.Load.End, {UM_GLOBAL.LoadVariables, "UpgradeMe", "UM_GLOBAL.LoadVar"})
table.insert(Event.System.Update.Begin, {UM_GLOBAL.Update, "UpgradeMe", "UM_GLOBAL.Update"})
table.insert(Event.Item.Slot, {UM_GLOBAL.SetOutfitDirty, "UpgradeMe", "UM_GLOBAL.ValidateOutfit"})
